@@ -36,7 +36,7 @@ from pydicti import odicti, dicti
 # meta data
 #----------------------------------------
 
-__version__ = 'madseq 0.4.2'
+__version__ = 'madseq 0.4.3'
 
 __all__ = [
     'Element', 'Sequence', 'Document',
@@ -126,7 +126,7 @@ class regex(object):
 
     # match single parameter=value argument and create groups:
     # (argument, assignment, value)
-    arg = Re(r',\s*(',identifier,r')\s*(:?=)\s*(',param,')')
+    arg = Re(r',\s*(',identifier,r')\s*(:?=)\s*(',param,')\s*')
 
     # match TEXT!COMMENT and return both parts as groups
     comment_split = Re(r'^([^!]*)(!.*)?$')
@@ -166,7 +166,7 @@ def format_value(value):
         elif isinstance(value, (tuple, list)):
             return '{' + ','.join(map(format_value, value)) + '}'
         else:
-            raise ValueError("Unknown data type: {!r}".format(value))
+            raise TypeError("Unknown data type: {0!r}".format(value))
 
 
 def format_safe(value):
@@ -215,6 +215,10 @@ class Value(object):
         """Return formatted value."""
         return self.expr
 
+    def __eq__(self, other):
+        """Compare the value."""
+        return other == self.value
+
     @classmethod
     def parse(cls, text, assign='='):
         """Parse MAD-X parameter input as any of the known Value types."""
@@ -238,7 +242,7 @@ def parse_number(text):
         try:
             return Decimal(text)
         except InvalidOperation:
-            raise ValueError("Not a floating point: {!r}".format(text))
+            raise ValueError("Not a floating point: {0!r}".format(text))
 
 
 @none_checked
@@ -247,7 +251,7 @@ def parse_string(text):
     try:
         return regex.is_string.match(str(text)).groups()[0]
     except AttributeError:
-        raise ValueError("Invalid string: {!r}".format(text))
+        raise ValueError("Invalid string: {0!r}".format(text))
 
 
 class Array(Value):
@@ -261,15 +265,15 @@ class Array(Value):
         """Parse a MAD-X array."""
         text = text.strip()
         if text[0] != '{':
-            raise ValueError("Invalid array: {!r}".format(text))
+            raise ValueError("Invalid array: {0!r}".format(text))
         if text[-1] != '}':
-            raise Exception("Ill-formed ARRAY: {!r}".format(text))
+            raise Exception("Ill-formed ARRAY: {0!r}".format(text))
         try:
             return cls([Value.parse(field.strip(), assign)
                         for field in text[1:-1].split(',')],
                        assign)
         except ValueError:
-            raise Exception("Ill-formed ARRAY: {!r}".format(text))
+            raise Exception("Ill-formed ARRAY: {0!r}".format(text))
 
     @property
     def expr(self):
@@ -319,7 +323,7 @@ class Identifier(Symbolic):
         try:
             return cls(regex.is_identifier.match(text).groups()[0], assign)
         except AttributeError:
-            raise ValueError("Invalid identifier: {!r}".format(text))
+            raise ValueError("Invalid identifier: {0!r}".format(text))
 
 
 class Composed(Symbolic):
@@ -348,6 +352,7 @@ class Composed(Symbolic):
 
 def parse_args(text):
     """Parse argument list into ordered dictionary."""
+    # TODO: use .split(',') to make expression simpler
     return odicti((key, Value.parse(val, assign))
                   for key,assign,val in regex.arg.findall(text or ''))
 
@@ -431,6 +436,10 @@ class Element(object):
         """Check whether key exists as argument in self or base."""
         return key in self.args or (self._base and key in self._base)
 
+    def __delitem__(self, key):
+        """Delete argument in self."""
+        del self.args[key]
+
     def __getitem__(self, key):
         """Get argument value from self or base."""
         try:
@@ -444,10 +453,6 @@ class Element(object):
         """Set argument value in self."""
         self.args[key] = val
 
-    def __delitem__(self, key):
-        """Delete argument in self."""
-        del self.args[key]
-
     def get(self, key, default=None):
         """Get argument value or default from self or base."""
         try:
@@ -575,8 +580,9 @@ class SequenceTransform(object):
         table.
         """
 
-        if isinstance(node, (Element, Sequence)):
-            defs[node.name] = node
+        if isinstance(node, Element):
+            defs[str(node.name)] = node
+            node._base = defs.get(node.type)
         if not isinstance(node, Sequence):
             return node
 
@@ -687,7 +693,7 @@ class ElementTransform(object):
         elif style == 'loop':
             self._distribution = self.uniform_slice_loop
         else:
-            raise ValueError("Unknown slicing style: {!r}".format(style))
+            raise ValueError("Unknown slicing style: {0!r}".format(style))
 
     def slice(self, elem, offset, refer):
         """
@@ -724,7 +730,7 @@ class ElementTransform(object):
         for slice_idx in range(slice_num):
             slice = elem.copy()
             slice['at'] = offset + (slice_idx + refer)*slice_len
-            if slice_num > 1:
+            if slice.name and slice_num > 1:
                 slice.name = elem.name + '..' + str(slice_idx)
             yield slice
 
@@ -752,6 +758,7 @@ class ElementTransform(object):
 
 def rescale_thick(elem, ratio):
     """Shrink/grow element size, while leaving the element type 'as is'."""
+    # TODO: implement this for all sorts of elements..
     if ratio == 1:
         return elem
     scaled = elem.copy()
@@ -767,26 +774,33 @@ def rescale_makethin(elem, ratio):
 
     NOTE: rescale_makethin is currently not recommended!  If you use it,
     you have to make sure, your slice length will be sufficiently small!
+
+    Furthermore, most MAD-X elements and parameters are not dealt with!
     """
+    # TODO: handle more elements (sextupole, ...?)
+    # TODO: check/modify more parameters
     base_type = elem.base_type
     if base_type not in ('sbend', 'quadrupole', 'solenoid'):
         return elem
     if base_type == 'solenoid':
         elem = elem.copy()
-        elem['ksi'] = elem['KS'] * ratio
+        elem['ksi'] = elem['KS'] * elem['L'] * ratio
         elem['lrad'] = elem['L'] * ratio
         elem['L'] = 0
         return elem
     elem = Element(elem.name, 'multipole', elem.all_args)
     if base_type == 'sbend':
-        elem['KNL'] = [elem['angle'] * ratio]
-        del elem['angle']
-        del elem['HGAP']
+        elem['KNL'] = [elem.pop('angle') * ratio]
+        elem.pop('HGAP', None)
     elif base_type == 'quadrupole':
-        elem['KNL'] = [0, elem['K1'] * elem['L']]
-        del elem['K1']
-    # replace L by LRAD property
-    elem['lrad'] = elem.pop('L', None)
+        if 'K1' in elem:
+            elem['KNL'] = [0, elem['K1'] * elem['L'] * ratio]
+            del elem['K1']
+        if 'K1S' in elem:
+            elem['KSL'] = [0, elem['K1S'] * elem['L'] * ratio]
+            del elem['K1S']
+    if 'L' in elem:
+        elem['lrad'] = elem.pop('L') * ratio
     return elem
 
 
@@ -958,7 +972,7 @@ class Document(list):
         elif fmt == 'madx':
             stream.write("\n".join(map(str, self._nodes)))
         else:
-            raise ValueError("Invalid format code: {!r}".format(fmt))
+            raise ValueError("Invalid format code: {0!r}".format(fmt))
 
 
 def main(argv=None):
